#ifndef __MM_HEADER_INCLUDED__
#define __MM_HEADER_INCLUDED__

#include <arch/mm.h>


/*
 * Page frame array
 */

#define PAGE_SIZE	(1UL<<PAGE_SHIFT)

/* There is one of these structures for every page
 * frame in the system. */
struct page {
	struct page *prev,*next;
	u_int32_t count;
	u_int32_t flags;
};


#define PG_reserved	0x01
#define PG_kmalloc	0x02

#define get_page(p) ((p)->count++)
#define put_page(p) ((p)->count--)

#define page_obj(p) ((p)->prev)

#define page_address(page) __va( ((page)-pfa) << PAGE_SHIFT )
#define virt_to_page(kaddr) (pfa + (__pa(kaddr) >> PAGE_SHIFT))


/* 
 * Buddy system 
 */

#define MAX_ORDER 10

extern struct page *pfa;
extern unsigned long nr_physpages;
extern unsigned long nr_freepages;

/* Members must be in the same order as struct page */
struct buddy {
	struct page *next,*prev;
	u_int32_t nr_free;
};

#define alloc_page() alloc_pages(0)
#define free_page(x) free_pages(x,0)

void free_pages(void *, int);
void *alloc_pages(int);
void buddy_init(void);

#endif /* __MM_HEADER_INCLUDED__ */
